from django.test import TestCase


class TestNh3FieldModelField(TestCase):
    """Test deprecated model field"""

    def test_init_raises_warning(self):
        with self.assertWarns(DeprecationWarning):
            from .deprecated_models import Nh3FieldContent  # noqa: F401



from __future__ import annotations

from collections.abc import Callable
from typing import Any

import nh3
from django import forms
from django.utils.safestring import mark_safe

from .utils import get_nh3_options


class Nh3Field(forms.CharField):
    """nh3 form field"""

    empty_values: list[Any] = [None, "", [], (), {}]

    def __init__(
        self,
        *args: Any,
        allowed_attributes: dict[str, set[str]] | None = None,
        allowed_attribute_filter: Callable[[str, str, str], str] | None = None,
        allowed_tags: set[str] | None = None,
        clean_content_tags: set[str] | None = None,
        empty_value: Any | None = "",
        generic_attribute_prefixes: set[str] | None = None,
        link_rel: str = "",
        set_tag_attribute_values: dict[str, dict[str, str]] | None = None,
        strip_comments: bool = False,
        tag_attribute_values: dict[str, dict[str, set[str]]] | None = None,
        url_schemes: set[str] | None = None,
        **kwargs: dict[Any, Any],
    ):
        super().__init__(*args, **kwargs)

        self.empty_value = empty_value
        self.nh3_options = get_nh3_options(
            attributes=allowed_attributes,
            attribute_filter=allowed_attribute_filter,
            clean_content_tags=clean_content_tags,
            generic_attribute_prefixes=generic_attribute_prefixes,
            link_rel=link_rel,
            set_tag_attribute_values=set_tag_attribute_values,
            strip_comments=strip_comments,
            tags=allowed_tags,
            tag_attribute_values=tag_attribute_values,
            url_schemes=url_schemes,
        )

    def to_python(self, value: Any) -> Any:
        """
        Strips any dodgy HTML tags from the input if the input value
        contains HTML

        Mark the return value as template safe if it contains HTML.
        """
        value = super().to_python(value)
        if value in self.empty_values:
            # Ensures that None is handled properly as an input
            return self.empty_value
        else:
            return mark_safe(nh3.clean(value, **self.nh3_options))



from unittest.mock import patch

from django.test import TestCase

from django_nh3.utils import get_nh3_default_options, normalize_nh3_options

from .constants import ALLOWED_ATTRIBUTES, ALLOWED_TAGS, STRIP_COMMENTS


class TestBleachOptions(TestCase):
    @patch(
        "django_nh3.utils.settings",
        NH3_ALLOWED_ATTRIBUTES=ALLOWED_ATTRIBUTES,
    )
    def test_custom_attrs(self, settings):
        nh3_args = get_nh3_default_options()
        self.assertEqual(nh3_args["attributes"], ALLOWED_ATTRIBUTES)

    @patch(
        "django_nh3.utils.settings",
        NH3_ALLOWED_TAGS=ALLOWED_TAGS,
    )
    def test_custom_tags(self, settings):
        nh3_args = get_nh3_default_options()
        self.assertEqual(nh3_args["tags"], ALLOWED_TAGS)

    @patch(
        "django_nh3.utils.settings",
        NH3_STRIP_COMMENTS=STRIP_COMMENTS,
    )
    def test_strip_comments(self, settings):
        nh3_args = get_nh3_default_options()
        self.assertEqual(nh3_args["strip_comments"], STRIP_COMMENTS)


def set_test_flag_true():
    return "set_test_flag_true"


class TestNormalizeNh3Options(TestCase):
    def test_unrecognized_keys_are_passed_through(self):
        self.assertEqual(normalize_nh3_options({"unknown": None}), {"unknown": None})
        self.assertEqual(normalize_nh3_options({"unknown": []}), {"unknown": []})
        self.assertEqual(normalize_nh3_options({"unknown": ()}), {"unknown": ()})
        self.assertEqual(normalize_nh3_options({"unknown": set()}), {"unknown": set()})

    def test_tags_clean_content_tags_generic_attribute_prefixes_and_url_schemes(self):
        for kwargs, expected in [
            (
                {"tags": ["one", "two", "three"]},
                {"tags": {"one", "two", "three"}},
            ),
            (
                {"clean_content_tags": ["two", "three", "four"]},
                {"clean_content_tags": {"two", "three", "four"}},
            ),
            (
                {"generic_attribute_prefixes": ["three", "four", "five"]},
                {"generic_attribute_prefixes": {"three", "four", "five"}},
            ),
            (
                {"url_schemes": ["four", "five", "six"]},
                {"url_schemes": {"four", "five", "six"}},
            ),
        ]:
            with self.subTest(kwargs=kwargs, expected=expected):
                self.assertDictEqual(
                    normalize_nh3_options(kwargs),
                    expected,
                )

    def test_attribute_filter(self):
        self.assertDictEqual(
            normalize_nh3_options({"attribute_filter": set_test_flag_true}),
            {"attribute_filter": set_test_flag_true},
        )

        # Make sure that the function is exactly what we expect
        result = normalize_nh3_options({"attribute_filter": set_test_flag_true})
        self.assertEqual(result["attribute_filter"](), "set_test_flag_true")

        self.assertDictEqual(
            normalize_nh3_options(
                {"attribute_filter": "tests.test_settings.set_test_flag_true"}
            ),
            {"attribute_filter": set_test_flag_true},
        )

        # Make sure that the function is exactly what we expect
        result = normalize_nh3_options({"attribute_filter": set_test_flag_true})
        self.assertEqual(result["attribute_filter"](), "set_test_flag_true")

    def test_strip_comments(self):
        for kwargs, expected in [
            (
                {"strip_comments": None},
                {"strip_comments": False},
            ),
            (
                {"strip_comments": []},
                {"strip_comments": False},
            ),
            (
                {"strip_comments": ""},
                {"strip_comments": False},
            ),
            (
                {"strip_comments": True},
                {"strip_comments": True},
            ),
            (
                {"strip_comments": "happy"},
                {"strip_comments": True},
            ),
        ]:
            with self.subTest(kwargs=kwargs, expected=expected):
                self.assertDictEqual(
                    normalize_nh3_options(kwargs),  # type: ignore[arg-type]
                    expected,
                )

    def test_link_rel(self):
        for kwargs, expected in [
            (
                {"link_rel": ""},
                {"link_rel": ""},
            ),
            (
                {"link_rel": "my string"},
                {"link_rel": "my string"},
            ),
            (
                {"link_rel": 0},
                {"link_rel": "0"},
            ),
            (
                {"link_rel": None},
                {"link_rel": None},
            ),
            (
                {"link_rel": True},
                {"link_rel": "True"},
            ),
            (
                {"link_rel": False},
                {"link_rel": "False"},
            ),
        ]:
            with self.subTest(kwargs=kwargs, expected=expected):
                self.assertDictEqual(
                    normalize_nh3_options(kwargs),  # type: ignore[arg-type]
                    expected,
                )

    def assertDataStructureEqual(self, left, right):
        if isinstance(left, set):
            self.assertIsInstance(right, set)
            self.assertSetEqual(left, right)
        elif isinstance(left, dict):
            self.assertIsInstance(right, dict)
            self.assertEqual(left.keys(), right.keys())
            for key in left.keys():
                self.assertIn(key, right.keys())
                self.assertDataStructureEqual(left[key], right[key])
        else:
            self.assertIsInstance(right, left.__class__)
            self.assertEqual(left, right)

    def test_tag_attribute_values(self):
        self.assertDataStructureEqual(
            normalize_nh3_options(
                {
                    "tag_attribute_values": {
                        "tag1": {
                            "attrA": ["A1", "A2", "A3"],
                            "attrB": ("B1", "B2", "B3"),
                        },
                        "tag2": {
                            "attrC": ["C1", "C2", "C3"],
                        },
                    },
                }
            ),
            {
                "tag_attribute_values": {
                    "tag1": {
                        "attrA": {"A1", "A2", "A3"},
                        "attrB": {"B1", "B2", "B3"},
                    },
                    "tag2": {
                        "attrC": {"C1", "C2", "C3"},
                    },
                },
            },
        )

    def test_set_tag_attribute_values(self):
        self.assertDataStructureEqual(
            normalize_nh3_options(
                {
                    "set_tag_attribute_values": {
                        "tag1": {
                            "attrA": "Avalue",
                            "attrB": "Bvalue",
                        },
                        "tag2": {
                            "attrC": "Cvalue",
                        },
                    },
                }
            ),
            {
                "set_tag_attribute_values": {
                    "tag1": {
                        "attrA": "Avalue",
                        "attrB": "Bvalue",
                    },
                    "tag2": {
                        "attrC": "Cvalue",
                    },
                },
            },
        )



from django.template import Context, Template
from django.test import TestCase


class TestBleachTemplates(TestCase):
    """Test template tags"""

    def test_bleaching(self):
        """Test that unsafe tags are sanitised"""
        context = Context(
            {"some_unsafe_content": '<script>alert("Hello World!")</script>'},
        )
        template_to_render = Template(
            "{% load nh3_tags %}" "{{ some_unsafe_content|nh3 }}"
        )
        rendered_template = template_to_render.render(context)
        self.assertEqual("", rendered_template)

    def test_bleaching_none(self):
        """Test that None is handled properly as an input"""
        context = Context({"none_value": None})
        template_to_render = Template("{% load nh3_tags %}" "{{ none_value|nh3 }}")
        rendered_template = template_to_render.render(context)
        self.assertEqual("None", rendered_template)

    def test_bleaching_tags(self):
        """Test provided tags are kept"""
        context = Context(
            {
                "some_unsafe_content": (
                    "<b><img src='' "
                    "onerror='alert(\\'hax\\')'>"
                    "I'm not trying to XSS you</b>"
                )
            }
        )
        template_to_render = Template(
            "{% load nh3_tags %}" '{{ some_unsafe_content|nh3:"img" }}'
        )
        rendered_template = template_to_render.render(context)
        self.assertInHTML('<img src="">I\'m not trying to XSS you', rendered_template)



import nh3
from django import template
from django.utils.safestring import SafeText, mark_safe

from django_nh3.utils import get_nh3_default_options

register = template.Library()


@register.filter(name="nh3")
def nh3_value(value: str | None, allowed_tags: str | None = None) -> SafeText | None:
    """
    Takes an input HTML value and sanitizes it utilizing nh3,
        returning a SafeText object that can be rendered by Django.

    Accepts an optional argument of allowed tags. Should be a comma delimited
        string (i.e. "img,span" or "img")
    """
    if value is None:
        return None

    nh3_args = get_nh3_default_options()
    if allowed_tags is not None:
        args = nh3_args.copy()
        args["tags"] = set(allowed_tags.split(","))
    else:
        args = nh3_args

    clean_value = nh3.clean(value, **args)
    return mark_safe(clean_value)



from __future__ import annotations

import warnings
from collections.abc import Callable
from typing import Any

import nh3
from django.db import models
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Expression, Model
from django.forms import Field as FormField
from django.utils.safestring import mark_safe
from typing_extensions import deprecated

from . import forms
from .utils import get_nh3_options


class Nh3FieldMixin:
    def __init__(
        self,
        *args: Any,
        allowed_attributes: dict[str, set[str]] | None = None,
        allowed_attribute_filter: Callable[[str, str, str], str] | None = None,
        allowed_tags: set[str] | None = None,
        clean_content_tags: set[str] | None = None,
        generic_attribute_prefixes: set[str] | None = None,
        link_rel: str = "",
        set_tag_attribute_values: dict[str, dict[str, str]] | None = None,
        strip_comments: bool = False,
        tag_attribute_values: dict[str, dict[str, set[str]]] | None = None,
        url_schemes: set[str] | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)

        self.nh3_options = get_nh3_options(
            attributes=allowed_attributes,
            attribute_filter=allowed_attribute_filter,
            clean_content_tags=clean_content_tags,
            generic_attribute_prefixes=generic_attribute_prefixes,
            link_rel=link_rel,
            set_tag_attribute_values=set_tag_attribute_values,
            strip_comments=strip_comments,
            tags=allowed_tags,
            tag_attribute_values=tag_attribute_values,
            url_schemes=url_schemes,
        )

    def formfield(
        self, form_class: FormField = forms.Nh3Field, **kwargs: Any
    ) -> FormField:
        """Makes the field for a ModelForm"""

        # If field doesn't have any choices add kwargs expected by Nh3Field.
        if not self.choices:  # type: ignore[attr-defined]
            kwargs.update(
                {
                    "allowed_attributes": self.nh3_options.get("attributes"),
                    "allowed_attribute_filter": self.nh3_options.get(
                        "attribute_filter"
                    ),
                    "allowed_tags": self.nh3_options.get("tags"),
                    "clean_content_tags": self.nh3_options.get("clean_content_tags"),
                    "generic_attribute_prefixes": self.nh3_options.get(
                        "generic_attribute_prefixes"
                    ),
                    "link_rel": self.nh3_options.get("link_rel"),
                    "max_length": self.max_length,  # type: ignore[attr-defined]
                    "required": not self.blank,  # type: ignore[attr-defined]
                    "set_tag_attribute_values": self.nh3_options.get(
                        "set_tag_attribute_values"
                    ),
                    "strip_comments": self.nh3_options.get("strip_comments"),
                    "tag_attribute_values": self.nh3_options.get(
                        "tag_attribute_values"
                    ),
                    "url_schemes": self.nh3_options.get("url_schemes"),
                }
            )

        return super().formfield(form_class=form_class, **kwargs)  # type: ignore[misc]

    def pre_save(self, model_instance: Model, add: bool) -> Any:
        data = getattr(model_instance, self.attname)  # type: ignore[attr-defined]
        if data is None:
            return data
        clean_value = nh3.clean(data, **self.nh3_options) if data else ""
        setattr(model_instance, self.attname, mark_safe(clean_value))  # type: ignore[attr-defined]
        return clean_value

    def from_db_value(
        self,
        value: Any,
        expression: Expression,
        connection: BaseDatabaseWrapper,
    ) -> Any:
        if value is None:
            return value
        # Values are sanitised before saving, so any value returned from the DB
        # is safe to render unescaped.
        return mark_safe(value)


class Nh3TextField(Nh3FieldMixin, models.TextField):
    pass


class Nh3CharField(Nh3FieldMixin, models.CharField):
    pass


@deprecated("Use Nh3TextField instead")
class Nh3Field(Nh3FieldMixin, models.TextField):
    """
    .. deprecated:: 0.2.0
    Use :class:`Nh3TextField` instead.
    """

    def __init__(self, *args, **kwargs) -> None:  # type: ignore[no-untyped-def]
        warnings.warn(
            "Nh3Field is deprecated and will be removed in a future version. "
            "Use Nh3TextField instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        super().__init__(*args, **kwargs)



from django.test import TestCase
from django.utils.safestring import SafeString

from django_nh3.forms import Nh3Field

# import urls to get coverage
from . import urls  # noqa: F401


class TestNh3Field(TestCase):
    def test_empty(self):
        """
        Test that the empty_value arg is returned for any input empty value
        """
        for requested_empty_value in ("", None):
            field = Nh3Field(empty_value=requested_empty_value)
            for empty_value in field.empty_values:
                self.assertEqual(field.to_python(empty_value), requested_empty_value)

    def test_return_type(self):
        """Test bleached values are SafeString objects"""
        field = Nh3Field()
        self.assertIsInstance(field.to_python("some text"), str)
        self.assertIsInstance(field.to_python("<h1>some text</h1>"), SafeString)

    def test_values(self):
        """Test bleached values are SafeString objects"""
        field = Nh3Field()
        self.assertEqual(field.to_python("some text"), "some text")
        self.assertEqual(field.to_python(" some text "), "some text")
        self.assertEqual(field.to_python("<h1>some text</h1>"), "some text")



import logging
from collections.abc import Callable
from typing import Any

from django.conf import settings
from django.utils.module_loading import import_string

logger = logging.getLogger(__name__)


def get_nh3_configured_default_options() -> dict[str, Any]:
    """
    Pull the django-nh3 settings similarly to how django-bleach handled them.

    Some django-bleach settings can be mapped to django-nh3 settings without
    any changes:

        BLEACH_ALLOWED_TAGS         -> NH3_ALLOWED_TAGS
        BLEACH_ALLOWED_ATTRIBUTES   -> NH3_ALLOWED_ATTRIBUTES
        BLEACH_STRIP_COMMENTS       -> NH3_STRIP_COMMENTS
        BLEACH_ALLOWED_PROTOCOLS    -> NH3_ALLOWED_URL_SCHEMES

    While other settings have no current support in nh3:

        BLEACH_ALLOWED_STYLES       -> There is no support for styling
        BLEACH_STRIP_TAGS           -> This is the default behavior of nh3

    """

    nh3_settings = {
        # Sets the tags that are allowed (eg: allowlist)
        # Ensure that no tags in this are also in NH3_CLEAN_CONTENT_TAGS or
        # NH3_ALLOWED_ATTRIBUTES
        "NH3_ALLOWED_TAGS": "tags",
        # Sets the tags whose contents will be completely removed from the
        # output (eg: blocklist)
        # Ensure that no tags in this are also in NH3_ALLOWED_TAGS or
        # NH3_ALLOWED_ATTRIBUTES
        # Default: script, style
        "NH3_ALLOWED_ATTRIBUTES": "attributes",
        # Sets the HTML attributes that are allowed on specific tags, * key
        # means the attributes are allowed on any tag (eg: allowlist)
        # Ensure that no tags in this are also in NH3_CLEAN_CONTENT_TAGS
        "NH3_CLEAN_CONTENT_TAGS": "clean_content_tags",
        # Dotted path to a callback that allows rewriting of all attributes.
        # The callback takes name of the element, attribute and its value.
        # Returns None to remove the attribute, or a value to use
        "NH3_ALLOWED_ATTRIBUTES_FILTER": "attribute_filter",
        # Configures the handling of HTML comments, defaults to True
        "NH3_STRIP_COMMENTS": "strip_comments",
        # Configures a rel attribute that will be added on links, defaults to
        # noopener noreferrer. To turn on rel-insertion, pass a space-separated
        # list. If rel is in the generic or tag attributes, this must be set to
        # None
        # Common rel values to include:
        # noopener
        # noreferrer
        # nofollow
        "NH3_LINK_REL": "link_rel",
        # Sets the prefix of attributes that are allowed on any tag
        "NH3_ALLOWED_GENERIC_ATTRIBUTE_PREFIXES": "generic_attribute_prefixes",
        # Sets the values of HTML attributes that are allowed on specific tags.
        # The value is structured as a map from tag names to a map from
        # attribute names to a set of attribute values. If a tag is not itself
        # whitelisted, adding entries to this map will do nothing.
        "NH3_ALLOWED_TAG_ATTRIBUTE_VALUES": "tag_attribute_values",
        # Sets the values of HTML attributes that are to be set on specific
        # tags. The value is structured as a map from tag names to a map from
        # attribute names to an attribute value. If a tag is not itself
        # whitelisted, adding entries to this map will do nothing.
        "NH3_SET_TAG_ATTRIBUTE_VALUES": "set_tag_attribute_values",
        # Sets the URL schemes permitted on href and src attributes
        "NH3_ALLOWED_URL_SCHEMES": "url_schemes",
    }

    return {
        kwarg: getattr(settings, setting_name)
        for setting_name, kwarg in nh3_settings.items()
        if hasattr(settings, setting_name)
    }


def normalize_nh3_options(  # noqa: C901, PLR0912
    options: dict[str, Any],
) -> dict[str, Any]:
    nh3_args: dict[str, Any] = {}
    for kwarg_name, kwarg_value in options.items():
        value = kwarg_value

        # Convert from general iterables to sets
        if kwarg_name in [
            "tags",
            "clean_content_tags",
            "generic_attribute_prefixes",
            "url_schemes",
        ]:
            value = set(value)

        elif kwarg_name == "attributes":
            copy_dict = value.copy()
            for tag, attributes in value.items():
                copy_dict[tag] = set(attributes)
            value = copy_dict

        elif kwarg_name == "attribute_filter":
            if callable(value):
                pass
            elif isinstance(value, str):
                value = import_string(value)

        elif kwarg_name == "strip_comments":
            value = bool(value)

        elif kwarg_name == "link_rel" and value is not None:
            value = str(value)

        elif kwarg_name == "tag_attribute_values":
            # The value is structured as a map from tag names to a map from
            # attribute names to a set of attribute values.
            allowed_tag_attr_dict: dict[str, dict[str, set[str]]] = {}
            for tag_name, attribute_dict in value.items():
                allowed_tag_attr_dict[tag_name] = {}
                for attr_name, attr_value in attribute_dict.items():
                    allowed_tag_attr_dict[tag_name][attr_name] = set(attr_value)
            value = allowed_tag_attr_dict

        elif kwarg_name == "set_tag_attribute_values":
            # The value is structured as a map from tag names to a map from
            # attribute names to an attribute value.
            set_tag_attr_dict: dict[str, dict[str, str]] = {}
            for tag_name, attribute_dict in value.items():
                set_tag_attr_dict[tag_name] = {}
                for attr_name, attr_value in attribute_dict.items():
                    set_tag_attr_dict[tag_name][attr_name] = str(attr_value)
            value = set_tag_attr_dict

        nh3_args[kwarg_name] = value

    return nh3_args


def get_nh3_default_options() -> dict[str, Any]:
    return normalize_nh3_options(get_nh3_configured_default_options())


def get_nh3_options(
    tags: set[str] | None = None,
    clean_content_tags: set[str] | None = None,
    attributes: dict[str, set[str]] | None = None,
    attribute_filter: Callable[[str, str, str], str] | None = None,
    strip_comments: bool = False,
    link_rel: str | None = None,
    generic_attribute_prefixes: set[str] | None = None,
    tag_attribute_values: dict[str, dict[str, set[str]]] | None = None,
    set_tag_attribute_values: dict[str, dict[str, str]] | None = None,
    url_schemes: set[str] | None = None,
) -> dict[str, Any]:
    defaults = get_nh3_configured_default_options()

    tags = tags or defaults.get("tags", None) or set()
    attributes = attributes or defaults.get("attributes", {})
    clean_content_tags = (
        clean_content_tags or defaults.get("clean_content_tags", None) or set()
    )
    attribute_filter = attribute_filter or defaults.get("attribute_filter", None)
    strip_comments = strip_comments or defaults.get("strip_comments", False)
    link_rel = link_rel or defaults.get("link_rel", None)
    generic_attribute_prefixes = (
        generic_attribute_prefixes
        or defaults.get("generic_attribute_prefixes", None)
        or set()
    )
    tag_attribute_values = (
        tag_attribute_values or defaults.get("tag_attribute_values", None) or {}
    )
    set_tag_attribute_values = (
        set_tag_attribute_values or defaults.get("set_tag_attribute_values", None) or {}
    )
    url_schemes = url_schemes or defaults.get("url_schemes", None) or set()

    return normalize_nh3_options(
        {
            "tags": tags,
            "clean_content_tags": clean_content_tags,
            "attributes": attributes,
            "attribute_filter": attribute_filter,
            "strip_comments": strip_comments,
            "link_rel": link_rel,
            "generic_attribute_prefixes": generic_attribute_prefixes,
            "tag_attribute_values": tag_attribute_values,
            "set_tag_attribute_values": set_tag_attribute_values,
            "url_schemes": url_schemes,
        }
    )



from django.test import TestCase
from django.utils.safestring import SafeString

from .forms import Nh3TextFieldContentModelForm, Nh3TextFieldNullableContentModelForm
from .models import Nh3TextFieldContent, Nh3TextFieldNullableContent


class TestNh3TextFieldModelField(TestCase):
    """Test model field"""

    def test_cleaning(self):
        """Test values are sanitized"""
        test_data = {
            "html_data": "<h1>Heading</h1>",
            "no_html": "Heading",
            "html_comment": "<!-- this is a comment -->",
        }
        expected_values = {
            "html_data": "Heading",
            "no_html": "Heading",
            "html_comment": "",
        }

        for key, value in test_data.items():
            obj = Nh3TextFieldContent.objects.create(content=value)
            self.assertEqual(obj.content, expected_values[key])

    def test_retrieved_values_are_template_safe(self):
        obj = Nh3TextFieldContent.objects.create(content="some content")
        obj.refresh_from_db()
        self.assertIsInstance(obj.content, SafeString)
        obj = Nh3TextFieldContent.objects.create(content="")
        obj.refresh_from_db()
        self.assertIsInstance(obj.content, SafeString)

    def test_saved_values_are_template_safe(self):
        obj = Nh3TextFieldContent(content="some content")
        obj.save()
        self.assertIsInstance(obj.content, SafeString)
        obj = Nh3TextFieldContent(content="")
        obj.save()
        self.assertIsInstance(obj.content, SafeString)

    def test_saved_none_values_are_none(self):
        obj = Nh3TextFieldContent(null_field=None)
        obj.save()
        self.assertIsNone(obj.null_field)


class TestNh3TextFieldNullableModelField(TestCase):
    """Test model field"""

    def test_cleaning(self):
        """Test values are sanitized"""
        test_data = {
            "none": None,
            "empty": "",
            "whitespaces": "   ",
            "linebreak": "\n",
        }
        expected_values = {
            "none": None,
            "empty": "",
            "whitespaces": "   ",
            "linebreak": "\n",
        }

        for key, value in test_data.items():
            obj = Nh3TextFieldNullableContent.objects.create(content=value)
            self.assertEqual(obj.content, expected_values[key])


class TestNh3TextFieldModelFormField(TestCase):
    """Test model form field"""

    def test_cleaning(self):
        """Test values are sanitized"""
        test_data = {
            "html_data": "<h1>Heading</h1>",
            "no_html": "Heading",
            "spacing": " Heading ",
        }
        expected_values = {
            "html_data": "Heading",
            "no_html": "Heading",
            "spacing": "Heading",
        }

        for key, value in test_data.items():
            form = Nh3TextFieldContentModelForm(data={"content": value})
            self.assertTrue(form.is_valid())
            obj = form.save()
            self.assertEqual(obj.content, expected_values[key])

    def test_stripped_comments(self):
        """Content field strips comments so ensure they aren't allowed"""

        self.assertFalse(
            Nh3TextFieldContentModelForm(
                data={"content": "<!-- this is a comment -->"}
            ).is_valid()
        )

    def test_field_choices(self):
        """Content field strips comments so ensure they aren't allowed"""
        test_data = dict(Nh3TextFieldNullableContent.CHOICES)

        for key, value in test_data.items():
            form = Nh3TextFieldNullableContentModelForm(data={"choice": key})
            self.assertTrue(form.is_valid())
            obj = form.save()
            self.assertEqual(obj.get_choice_display(), value)
